module approx_mult (
    input  logic signed [ 7:0] i_a,
    input  logic signed [ 7:0] i_b,
    output logic signed [15:0] o_z
);


reg [3:0] S;
reg [3:0] E;

reg [8:0] pp0,pp1,pp2,pp3;


reg [11:0] partial_product_0;
reg [10:0] partial_product_1, partial_product_2;
reg [9:0] partial_product_3;
reg [15:0] result;
reg [2:0] group[0:3];

// Stage 1 generate Ei
approx_Ei_find e(
  .S(S),
  .i_a(i_a),
  .E(E),
  .group(group)
);



// stage 2 generate Si
approx_Si_find s(
  .i_b(i_b),
  .S(S),
  .group(group)
);



// stage 3 
approx_partial_product pp_0(.i_a(i_a), .radix_option(group[0]), .partial_p(pp0));
approx_partial_product pp_1(.i_a(i_a), .radix_option(group[1]), .partial_p(pp1));
approx_partial_product pp_2(.i_a(i_a), .radix_option(group[2]), .partial_p(pp2));
approx_partial_product pp_3(.i_a(i_a), .radix_option(group[3]), .partial_p(pp3));



// stage 4
approx_model approx1(
 .pp0(pp0), .pp1(pp1), .pp2(pp2), .pp3(pp3),
 .E(E),
 .pp0_out(partial_product_0),
 .pp1_out(partial_product_1),
 .pp2_out(partial_product_2),
 .pp3_out(partial_product_3)
);


// stage 5
approx_final_result r(
  .partial_product_0(partial_product_0),
  .partial_product_1(partial_product_1),
  .partial_product_2(partial_product_2),
  .partial_product_3(partial_product_3),
  .S(S),
  .total(o_z)
);

endmodule
